@@ -1,7 +1,5 @@
 use std::borrow::Cow;
 
-use anyhow::Result;
-
 use pep508_rs::VerbatimUrl;
 use uv_normalize::PackageName;
 
@@ -9,9 +7,9 @@ use crate::error::Error;
 use crate::{
     BuiltDist, CachedDirectUrlDist, CachedDist, CachedRegistryDist, DirectUrlBuiltDist,
     DirectUrlSourceDist, Dist, DistributionId, GitSourceDist, InstalledDirectUrlDist,
-    InstalledDist, InstalledRegistryDist, InstalledVersion, LocalDist, PackageId, PathBuiltDist,
-    PathSourceDist, RegistryBuiltDist, RegistrySourceDist, ResourceId, SourceDist, VersionId,
-    VersionOrUrlRef,
+    InstalledDist, InstalledEggInfoDirectory, InstalledEggInfoFile, InstalledLegacyEditable,
+    InstalledRegistryDist, InstalledVersion, LocalDist, PackageId, PathBuiltDist, PathSourceDist,
+    RegistryBuiltWheel, RegistrySourceDist, ResourceId, SourceDist, VersionId, VersionOrUrlRef,
 };
 
 pub trait Name {
@@ -191,6 +189,24 @@ impl std::fmt::Display for InstalledRegistryDist {
     }
 }
 
+impl std::fmt::Display for InstalledEggInfoFile {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}{}", self.name(), self.installed_version())
+    }
+}
+
+impl std::fmt::Display for InstalledEggInfoDirectory {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}{}", self.name(), self.installed_version())
+    }
+}
+
+impl std::fmt::Display for InstalledLegacyEditable {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}{}", self.name(), self.installed_version())
+    }
+}
+
 impl std::fmt::Display for PathBuiltDist {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}{}", self.name(), self.version_or_url())
@@ -203,7 +219,7 @@ impl std::fmt::Display for PathSourceDist {
     }
 }
 
-impl std::fmt::Display for RegistryBuiltDist {
+impl std::fmt::Display for RegistryBuiltWheel {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}{}", self.name(), self.version_or_url())
     }
