@@ -2,12 +2,11 @@ use std::collections::{BTreeMap, BTreeSet};
 use std::fmt::Formatter;
 use std::sync::Arc;
 
-use indexmap::IndexMap;
+use dashmap::DashMap;
 use pubgrub::range::Range;
 use pubgrub::report::{DefaultStringReporter, DerivationTree, External, Reporter};
 use rustc_hash::{FxHashMap, FxHashSet};
 
-use dashmap::DashMap;
 use distribution_types::{BuiltDist, IndexLocations, InstalledDist, SourceDist};
 use pep440_rs::Version;
 use pep508_rs::Requirement;
@@ -15,7 +14,9 @@ use uv_normalize::PackageName;
 
 use crate::candidate_selector::CandidateSelector;
 use crate::dependency_provider::UvDependencyProvider;
-use crate::pubgrub::{PubGrubPackage, PubGrubPackageInner, PubGrubReportFormatter};
+use crate::pubgrub::{
+    PubGrubPackage, PubGrubPackageInner, PubGrubReportFormatter, PubGrubSpecifierError,
+};
 use crate::python_requirement::PythonRequirement;
 use crate::resolver::{
     FxOnceMap, IncompletePackage, UnavailablePackage, UnavailableReason, VersionsResponse,
@@ -44,8 +45,8 @@ pub enum ResolveError {
         metadata: PackageName,
     },
 
-    #[error("~= operator requires at least two release segments: `{0}`")]
-    InvalidTildeEquals(pep440_rs::VersionSpecifier),
+    #[error(transparent)]
+    PubGrubSpecifier(#[from] PubGrubSpecifierError),
 
     #[error("Requirements contain conflicting URLs for package `{0}`:\n- {1}\n- {2}")]
     ConflictingUrlsDirect(PackageName, String, String),
@@ -110,7 +111,7 @@ impl<T> From<tokio::sync::mpsc::error::SendError<T>> for ResolveError {
 
 /// Given a [`DerivationTree`], collapse any [`External::FromDependencyOf`] incompatibilities
 /// wrap an [`PubGrubPackageInner::Extra`] package.
-fn collapse_extra_proxies(
+fn collapse_proxies(
     derivation_tree: &mut DerivationTree<PubGrubPackage, Range<Version>, UnavailableReason>,
 ) {
     match derivation_tree {
@@ -123,20 +124,32 @@ fn collapse_extra_proxies(
                 (
                     DerivationTree::External(External::FromDependencyOf(package, ..)),
                     ref mut cause,
-                ) if matches!(&**package, PubGrubPackageInner::Extra { .. }) => {
-                    collapse_extra_proxies(cause);
+                ) if matches!(
+                    &**package,
+                    PubGrubPackageInner::Extra { .. }
+                        | PubGrubPackageInner::Marker { .. }
+                        | PubGrubPackageInner::Dev { .. }
+                ) =>
+                {
+                    collapse_proxies(cause);
                     *derivation_tree = cause.clone();
                 }
                 (
                     ref mut cause,
                     DerivationTree::External(External::FromDependencyOf(package, ..)),
-                ) if matches!(&**package, PubGrubPackageInner::Extra { .. }) => {
-                    collapse_extra_proxies(cause);
+                ) if matches!(
+                    &**package,
+                    PubGrubPackageInner::Extra { .. }
+                        | PubGrubPackageInner::Marker { .. }
+                        | PubGrubPackageInner::Dev { .. }
+                ) =>
+                {
+                    collapse_proxies(cause);
                     *derivation_tree = cause.clone();
                 }
                 _ => {
-                    collapse_extra_proxies(Arc::make_mut(&mut derived.cause1));
-                    collapse_extra_proxies(Arc::make_mut(&mut derived.cause2));
+                    collapse_proxies(Arc::make_mut(&mut derived.cause1));
+                    collapse_proxies(Arc::make_mut(&mut derived.cause2));
                 }
             }
         }
@@ -154,12 +167,12 @@ impl From<pubgrub::error::PubGrubError<UvDependencyProvider>> for ResolveError {
             }
             pubgrub::error::PubGrubError::Failure(inner) => Self::Failure(inner),
             pubgrub::error::PubGrubError::NoSolution(mut derivation_tree) => {
-                collapse_extra_proxies(&mut derivation_tree);
+                collapse_proxies(&mut derivation_tree);
 
                 Self::NoSolution(NoSolutionError {
                     derivation_tree,
                     // The following should be populated before display for the best error messages
-                    available_versions: IndexMap::default(),
+                    available_versions: FxHashMap::default(),
                     selector: None,
                     python_requirement: None,
                     index_locations: None,
@@ -181,7 +194,7 @@ impl From<pubgrub::error::PubGrubError<UvDependencyProvider>> for ResolveError {
 #[derive(Debug)]
 pub struct NoSolutionError {
     derivation_tree: DerivationTree<PubGrubPackage, Range<Version>, UnavailableReason>,
-    available_versions: IndexMap<PubGrubPackage, BTreeSet<Version>>,
+    available_versions: FxHashMap<PubGrubPackage, BTreeSet<Version>>,
     selector: Option<CandidateSelector>,
     python_requirement: Option<PythonRequirement>,
     index_locations: Option<IndexLocations>,
@@ -227,12 +240,14 @@ impl NoSolutionError {
         visited: &FxHashSet<PackageName>,
         package_versions: &FxOnceMap<PackageName, Arc<VersionsResponse>>,
     ) -> Self {
-        let mut available_versions = IndexMap::default();
+        let mut available_versions = FxHashMap::default();
         for package in self.derivation_tree.packages() {
             match &**package {
-                PubGrubPackageInner::Root(_) => {}
-                PubGrubPackageInner::Python(_) => {}
+                PubGrubPackageInner::Root { .. } => {}
+                PubGrubPackageInner::Python { .. } => {}
+                PubGrubPackageInner::Marker { .. } => {}
                 PubGrubPackageInner::Extra { .. } => {}
+                PubGrubPackageInner::Dev { .. } => {}
                 PubGrubPackageInner::Package { name, .. } => {
                     // Avoid including available versions for packages that exist in the derivation
                     // tree, but were never visited during resolution. We _may_ have metadata for
