@@ -16,17 +16,22 @@ pub(crate) use pip::list::pip_list;
 pub(crate) use pip::show::pip_show;
 pub(crate) use pip::sync::pip_sync;
 pub(crate) use pip::uninstall::pip_uninstall;
+pub(crate) use project::add::add;
 pub(crate) use project::lock::lock;
+pub(crate) use project::remove::remove;
 pub(crate) use project::run::run;
 pub(crate) use project::sync::sync;
 #[cfg(feature = "self-update")]
 pub(crate) use self_update::self_update;
 pub(crate) use tool::run::run as run_tool;
+pub(crate) use toolchain::find::find as toolchain_find;
+pub(crate) use toolchain::install::install as toolchain_install;
+pub(crate) use toolchain::list::list as toolchain_list;
 use uv_cache::Cache;
 use uv_fs::Simplified;
 use uv_installer::compile_tree;
-use uv_interpreter::PythonEnvironment;
 use uv_normalize::PackageName;
+use uv_toolchain::PythonEnvironment;
 pub(crate) use venv::venv;
 pub(crate) use version::version;
 
@@ -35,10 +40,11 @@ use crate::printer::Printer;
 mod cache_clean;
 mod cache_dir;
 mod cache_prune;
-mod pip;
+pub(crate) mod pip;
 mod project;
 pub(crate) mod reporters;
 mod tool;
+mod toolchain;
 
 #[cfg(feature = "self-update")]
 mod self_update;
@@ -134,7 +140,7 @@ pub(super) async fn compile_bytecode(
     let start = std::time::Instant::now();
     let mut files = 0;
     for site_packages in venv.site_packages() {
-        files += compile_tree(site_packages, venv.python_executable(), cache.root())
+        files += compile_tree(&site_packages, venv.python_executable(), cache.root())
             .await
             .with_context(|| {
                 format!(
