@@ -21,8 +21,8 @@ use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    Concurrency, ConfigSettings, Constraints, ExtrasSpecification, IndexStrategy, NoBinary,
-    NoBuild, Overrides, PreviewMode, SetupPyStrategy, Upgrade,
+    BuildOptions, Concurrency, ConfigSettings, Constraints, ExtrasSpecification, IndexStrategy,
+    Overrides, PreviewMode, SetupPyStrategy, Upgrade,
 };
 use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
@@ -80,7 +80,7 @@ pub(crate) async fn pip_compile(
     config_settings: ConfigSettings,
     connectivity: Connectivity,
     no_build_isolation: bool,
-    no_build: NoBuild,
+    build_options: BuildOptions,
     python_version: Option<PythonVersion>,
     python_platform: Option<TargetTriple>,
     exclude_newer: Option<ExcludeNewer>,
@@ -122,8 +122,8 @@ pub(crate) async fn pip_compile(
         extra_index_urls,
         no_index,
         find_links,
-        no_binary: _,
-        no_build: specified_no_build,
+        no_binary,
+        no_build,
     } = RequirementsSpecification::from_sources(
         requirements,
         constraints,
@@ -253,11 +253,14 @@ pub(crate) async fn pip_compile(
     let preferences = read_requirements_txt(output_file, &upgrade).await?;
     let git = GitResolver::default();
 
+    // Combine the `--no-binary` and `--no-build` flags from the requirements files.
+    let build_options = build_options.combine(no_binary, no_build);
+
     // Resolve the flat indexes from `--find-links`.
     let flat_index = {
         let client = FlatIndexClient::new(&client, &cache);
         let entries = client.fetch(index_locations.flat_index()).await?;
-        FlatIndex::from_entries(entries, Some(&tags), &hasher, &no_build, &NoBinary::None)
+        FlatIndex::from_entries(entries, Some(&tags), &hasher, &build_options)
     };
 
     // Track in-flight downloads, builds, etc., across resolutions.
@@ -272,9 +275,6 @@ pub(crate) async fn pip_compile(
         BuildIsolation::Isolated
     };
 
-    // Combine the `--no-build` flags.
-    let no_build = no_build.combine(specified_no_build);
-
     let build_dispatch = BuildDispatch::new(
         &client,
         &cache,
@@ -288,8 +288,7 @@ pub(crate) async fn pip_compile(
         &config_settings,
         build_isolation,
         link_mode,
-        &no_build,
-        &NoBinary::None,
+        &build_options,
         concurrency,
         preview,
     )
